@xbot.xfunction
@bot.command(name='add')
async def add(ctx):
    """__GENERATED__ cross platform generated function"""
    try:
        message = ctx.message.content
        (_, left, right) = message.split(' ')
        result = (int(left) + int(right))
        await ctx.send(result)
    except (IndexError, ValueError):
        await ctx.send('Usage: /add <left> <right>')

@xbot.xfunction
@bot.command(name='subtract')
async def subtract(ctx):
    """__GENERATED__ cross platform generated function"""
    try:
        message = ctx.message.content
        (_, left, right) = message.split(' ')
        result = (int(left) - int(right))
        await ctx.send(result)
    except (IndexError, ValueError):
        await ctx.send('Usage: /subtract <left> <right>')

@xbot.xfunction
@bot.command(name='echo')
async def echo(ctx):
    """__GENERATED__ cross platform generated function"""
    await ctx.send(ctx.message.content)


"""e2e translation test"""
import asyncio
import pytest

import xbot
import xbot.__main__
import xbot.constants

def test_e2e__telegram_to_discord():
    sourcecode_filename = "./tests/original_code.py"
    xbot.translate.translate(sourcecode_filename)

    expected_code = ""
    with open("./tests/translated_code__expected.py","r") as f:
        expected_code = f.read()

    translated_code = ""
    with open(xbot.constants.DEFAULT_OUTPUT_FILENAME, "r") as f:
        translated_code = f.read()

    assert expected_code == translated_code

@xbot.xfunction
def example_function(a):
    return a + 1

@xbot.xfunction
async def example_async_function(a):
    await asyncio.sleep(1)
    return a + 1

@pytest.mark.asyncio
async def test__xfunction():
    """
    we just want to test that the decorator
    compiles and doesn't break anything
    since is used only for parsing
    """
    assert example_function(1) == 2
    assert asyncio.iscoroutinefunction(example_async_function)
    temp = await example_async_function(1)
    assert temp == 2



import functools
import asyncio

def xfunction(func):
    """
    This is a decorator to add in front of functions
    you want to translate. It doesn't do anything by
    itself but it's used by the parser to figure out
    whether to translate this function.

    Need to support both functions and coroutines,
    see https://stackoverflow.com/questions/44169998/
    """
    @functools.wraps(func)
    def wrapper(*args, **kwds):
        return func(*args, **kwds)

    @functools.wraps(func)
    async def async_wrapper(*args, **kwds):
        return (await func(*args, **kwds))

    if asyncio.iscoroutinefunction(func):
        return async_wrapper
    else:
        return wrapper




"""here is the source (to be translated) code"""
import xbot

@xbot.xfunction
def add(update, context):
    try:
        message = update.message.text
        _, left, right = message.split(" ")
        result = int(left) + int(right)
        update.message.reply_text(result)
    except (IndexError, ValueError):
        update.message.reply_text('Usage: /add <left> <right>')

@xbot.xfunction
def subtract(update, context):
    try:
        message = update.message.text
        _, left, right = message.split(" ")
        result = int(left) - int(right)
        update.message.reply_text(result)
    except (IndexError, ValueError):
        update.message.reply_text('Usage: /subtract <left> <right>')

@xbot.xfunction
def echo(update, context):
    update.message.reply_text(update.message.text)

def echo_dont_translate(update, context):
    update.message.reply_text(update.message.text)
